@@ -1,16 +1,18 @@
+(* NOTE: when modifying this file, please also check whether
+   rng/miou/pfortuna.ml needs to be updated. *)
+
 open Mirage_crypto
 open Mirage_crypto.Uncommon
 
-module AES_CTR = Cipher_block.AES.CTR
-
 module SHAd256 = struct
-  open Hash
+  open Digestif
   type t = SHA256.t
+  type ctx = SHA256.ctx
   let empty     = SHA256.empty
-  let get t     = SHA256.(get t |> digest)
-  let digest x  = SHA256.(digest x |> digest)
-  let digesti i = SHA256.(digesti i |> digest)
-  let feedi     = SHA256.feedi
+  let get t     = SHA256.(get t |> to_raw_string |> digest_string |> to_raw_string)
+  let digest x  = SHA256.(digest_string x |> to_raw_string |> digest_string |> to_raw_string)
+  let digesti i = SHA256.(digesti_string i |> to_raw_string |> digest_string |> to_raw_string)
+  let feedi     = SHA256.feedi_string
 end
 
 let block = 16
@@ -24,10 +26,10 @@ let pools = 32
 
 (* XXX Locking!! *)
 type g =
-  { mutable ctr    : AES_CTR.ctr
-  ; mutable secret : Cstruct.t
-  ; mutable key    : AES_CTR.key
-  ; pools          : SHAd256.t array
+  { mutable ctr    : AES.CTR.ctr
+  ; mutable secret : string
+  ; mutable key    : AES.CTR.key
+  ; pools          : SHAd256.ctx array
   ; mutable pool0_size : int
   ; mutable reseed_count : int
   ; mutable last_reseed : int64
@@ -35,10 +37,10 @@ type g =
   }
 
 let create ?time () =
-  let k = Cstruct.create 32 in
+  let k = String.make 32 '\x00' in
   { ctr    = (0L, 0L)
   ; secret = k
-  ; key    = AES_CTR.of_secret k
+  ; key    = AES.CTR.of_secret k
   ; pools  = Array.make pools SHAd256.empty
   ; pool0_size = 0
   ; reseed_count = 0
@@ -53,25 +55,24 @@ let seeded ~g =
 (* XXX We might want to erase the old key. *)
 let set_key ~g sec =
   g.secret <- sec ;
-  g.key    <- AES_CTR.of_secret sec
+  g.key    <- AES.CTR.of_secret sec
 
 let reseedi ~g iter =
   set_key ~g @@ SHAd256.digesti (fun f -> f g.secret; iter f);
-  g.ctr <- AES_CTR.add_ctr g.ctr 1L
+  g.ctr <- AES.CTR.add_ctr g.ctr 1L
 
 let iter1 a     f = f a
 
 let reseed ~g cs = reseedi ~g (iter1 cs)
 
-let generate_rekey ~g bytes =
-  let b  = bytes // block + 2 in
+let generate_rekey ~g buf ~off len =
+  let b  = len // block + 2 in
   let n  = b * block in
-  let r  = AES_CTR.stream ~key:g.key ~ctr:g.ctr n in
-  let r1 = Cstruct.sub r 0 bytes
-  and r2 = Cstruct.sub r (n - 32) 32 in
+  let r  = AES.CTR.stream ~key:g.key ~ctr:g.ctr n in
+  Bytes.unsafe_blit_string r 0 buf off len;
+  let r2 = String.sub r (n - 32) 32 in
   set_key ~g r2 ;
-  g.ctr <- AES_CTR.add_ctr g.ctr (Int64.of_int b);
-  r1
+  g.ctr <- AES.CTR.add_ctr g.ctr (Int64.of_int b)
 
 let add_pool_entropy g =
   if g.pool0_size > min_pool_size then
@@ -93,24 +94,26 @@ let add_pool_entropy g =
       done
     end
 
-let generate ~g bytes =
+let generate_into ~g buf ~off len =
   add_pool_entropy g;
   if not (seeded ~g) then raise Rng.Unseeded_generator ;
-  let rec chunk acc = function
-    | i when i <= 0 -> acc
-    | n -> let n' = imin n 0x10000 in
-           chunk (generate_rekey ~g n' :: acc) (n - n') in
-  Cstruct.concat @@ chunk [] bytes
-
-let _buf = Cstruct.create_unsafe 2
+  let rec chunk off = function
+    | i when i <= 0 -> ()
+    | n ->
+      let n' = imin n 0x10000 in
+      generate_rekey ~g buf ~off n';
+      chunk (off + n') (n - n')
+  in
+  chunk off len
 
 let add ~g (source, _) ~pool data =
-    let pool   = pool land (pools - 1)
-    and source = source land 0xff in
-    Cstruct.set_uint8 _buf 0 source;
-    Cstruct.set_uint8 _buf 1 (Cstruct.length data);
-    g.pools.(pool) <- SHAd256.feedi g.pools.(pool) (iter2 _buf data);
-    if pool = 0 then g.pool0_size <- g.pool0_size + Cstruct.length data
+  let buf = Bytes.create 2
+  and pool   = pool land (pools - 1)
+  and source = source land 0xff in
+  Bytes.set_uint8 buf 0 source;
+  Bytes.set_uint8 buf 1 (String.length data);
+  g.pools.(pool) <- SHAd256.feedi g.pools.(pool) (iter2 (Bytes.unsafe_to_string buf) data);
+  if pool = 0 then g.pool0_size <- g.pool0_size + String.length data
 
 (* XXX
  * Schneier recommends against using generator-imposed pool-seeding schedule
@@ -118,6 +121,6 @@ let add ~g (source, _) ~pool data =
  *)
 let accumulate ~g source =
   let pool = ref 0 in
-  `Acc (fun cs ->
-    add ~g source ~pool:!pool cs ;
+  `Acc (fun buf ->
+    add ~g source ~pool:!pool buf ;
     incr pool)
