@@ -17,14 +17,8 @@
 #define _S_1111 0x55
 #define _S_0000 0x00
 
-/*
- * RKs are currently aligned from the C side on access. Would be better to
- * allocate and pass them in pre-aligned.
- *
- * XXX Get rid of the correction here.
- */
 static int _mc_aesni_rk_size (uint8_t rounds) {
-  return (rounds + 1) * 16 + 15;
+  return (rounds + 1) * 16;
 }
 
 #if defined(__x86_64__) || defined(_WIN64)
@@ -61,9 +55,10 @@ static inline void __pack (__m128i *o1, __m128i *o2, __m128i r1, __m128i r2, __m
 #endif
 
 static inline void _mc_aesni_derive_e_key (const uint8_t *key, uint8_t *rk0, uint8_t rounds) {
-
-  __m128i *rk = __rk (rk0);
+  __m128i schedule[15 + 1];
+  __m128i *rk = __rk (schedule);
   __m128i temp1, temp2;
+  int i;
 
   switch (rounds) {
     case 10:
@@ -142,24 +137,26 @@ static inline void _mc_aesni_derive_e_key (const uint8_t *key, uint8_t *rk0, uin
 
     default:
       ;
+  };
+
+  for (i = 0; i <= rounds; i++) {
+    _mm_storeu_si128((__m128i*) rk0 + i, rk[i]);
   }
 }
 
-static inline void _mc_aesni_invert_e_key (const uint8_t *rk0, uint8_t *kr0, uint8_t rounds) {
+static inline void _mc_aesni_invert_e_key (const uint8_t *rk1, uint8_t *kr0, uint8_t rounds) {
 
-  __m128i *rk1 = __rk (rk0),
-          *kr  = __rk (kr0),
-          rk[15];
+  __m128i rk[15];
 
   for (uint8_t i = 0; i <= rounds; i++)
-    rk[i] = rk1[i];
+    rk[i] = _mm_loadu_si128 ((__m128i*) rk1 + i);
 
-  kr[0] = rk[rounds];
+  _mm_storeu_si128((__m128i*) kr0 + 0, rk[rounds]);
 
   for (uint8_t i = 1; i < rounds; i++)
-    kr[i] = _mm_aesimc_si128 (rk[rounds - i]);
+    _mm_storeu_si128((__m128i*) kr0 + i, _mm_aesimc_si128 (rk[rounds - i]));
 
-  kr[rounds] = rk[0];
+  _mm_storeu_si128((__m128i*) kr0 + rounds, rk[0]);
 }
 
 static void _mc_aesni_derive_d_key (const uint8_t *key, uint8_t *kr, uint8_t rounds, uint8_t *rk) {
@@ -174,28 +171,36 @@ static void _mc_aesni_derive_d_key (const uint8_t *key, uint8_t *kr, uint8_t rou
 static inline void _mc_aesni_enc (const uint8_t src[16], uint8_t dst[16], const uint8_t *rk0, uint8_t rounds) {
 
   __m128i r   = _mm_loadu_si128 ((__m128i*) src),
-          *rk = __rk (rk0);
+          rk;
 
-  r = _mm_xor_si128 (r, rk[0]);
+  rk = _mm_loadu_si128 ((__m128i*) rk0 + 0);
+  r = _mm_xor_si128 (r, rk);
 
-  for (uint8_t i = 1; i < rounds; i++)
-    r = _mm_aesenc_si128 (r, rk[i]);
+  for (uint8_t i = 1; i < rounds; i++) {
+    rk = _mm_loadu_si128 ((__m128i*) rk0 + i);
+    r = _mm_aesenc_si128 (r, rk);
+  }
 
-  r = _mm_aesenclast_si128 (r, rk[rounds]);
+  rk = _mm_loadu_si128 ((__m128i*) rk0 + rounds);
+  r = _mm_aesenclast_si128 (r, rk);
   _mm_storeu_si128 ((__m128i*) dst, r);
 }
 
 static inline void _mc_aesni_dec (const uint8_t src[16], uint8_t dst[16], const uint8_t *rk0, uint8_t rounds) {
 
   __m128i r   = _mm_loadu_si128 ((__m128i*) src),
-          *rk = __rk (rk0);
+          rk;
 
-  r = _mm_xor_si128 (r, rk[0]);
+  rk = _mm_loadu_si128 ((__m128i*) rk0 + 0);
+  r = _mm_xor_si128 (r, rk);
 
-  for (uint8_t i = 1; i < rounds; i++)
-    r = _mm_aesdec_si128 (r, rk[i]);
+  for (uint8_t i = 1; i < rounds; i++) {
+    rk = _mm_loadu_si128 ((__m128i*) rk0 + i);
+    r = _mm_aesdec_si128 (r, rk);
+  }
 
-  r = _mm_aesdeclast_si128 (r, rk[rounds]);
+  rk = _mm_loadu_si128 ((__m128i*) rk0 + rounds);
+  r = _mm_aesdeclast_si128 (r, rk);
   _mm_storeu_si128 ((__m128i*) dst, r);
 }
 
@@ -203,7 +208,7 @@ static inline void _mc_aesni_enc8 (const uint8_t src[128], uint8_t dst[128], con
 
   __m128i *in  = (__m128i*) src,
           *out = (__m128i*) dst,
-          *rk  = __rk (rk0);
+          rk;
 
   __m128i r0 = _mm_loadu_si128 (in    ),
           r1 = _mm_loadu_si128 (in + 1),
@@ -214,34 +219,37 @@ static inline void _mc_aesni_enc8 (const uint8_t src[128], uint8_t dst[128], con
           r6 = _mm_loadu_si128 (in + 6),
           r7 = _mm_loadu_si128 (in + 7);
 
-  r0 = _mm_xor_si128 (r0, rk[0]);
-  r1 = _mm_xor_si128 (r1, rk[0]);
-  r2 = _mm_xor_si128 (r2, rk[0]);
-  r3 = _mm_xor_si128 (r3, rk[0]);
-  r4 = _mm_xor_si128 (r4, rk[0]);
-  r5 = _mm_xor_si128 (r5, rk[0]);
-  r6 = _mm_xor_si128 (r6, rk[0]);
-  r7 = _mm_xor_si128 (r7, rk[0]);
+  rk = _mm_loadu_si128 ((__m128i*) rk0 + 0);
+  r0 = _mm_xor_si128 (r0, rk);
+  r1 = _mm_xor_si128 (r1, rk);
+  r2 = _mm_xor_si128 (r2, rk);
+  r3 = _mm_xor_si128 (r3, rk);
+  r4 = _mm_xor_si128 (r4, rk);
+  r5 = _mm_xor_si128 (r5, rk);
+  r6 = _mm_xor_si128 (r6, rk);
+  r7 = _mm_xor_si128 (r7, rk);
 
   for (uint8_t i = 1; i < rounds; i++) {
-    r0 = _mm_aesenc_si128 (r0, rk[i]);
-    r1 = _mm_aesenc_si128 (r1, rk[i]);
-    r2 = _mm_aesenc_si128 (r2, rk[i]);
-    r3 = _mm_aesenc_si128 (r3, rk[i]);
-    r4 = _mm_aesenc_si128 (r4, rk[i]);
-    r5 = _mm_aesenc_si128 (r5, rk[i]);
-    r6 = _mm_aesenc_si128 (r6, rk[i]);
-    r7 = _mm_aesenc_si128 (r7, rk[i]);
+    rk = _mm_loadu_si128 ((__m128i*) rk0 + i);
+    r0 = _mm_aesenc_si128 (r0, rk);
+    r1 = _mm_aesenc_si128 (r1, rk);
+    r2 = _mm_aesenc_si128 (r2, rk);
+    r3 = _mm_aesenc_si128 (r3, rk);
+    r4 = _mm_aesenc_si128 (r4, rk);
+    r5 = _mm_aesenc_si128 (r5, rk);
+    r6 = _mm_aesenc_si128 (r6, rk);
+    r7 = _mm_aesenc_si128 (r7, rk);
   }
 
-  r0 = _mm_aesenclast_si128 (r0, rk[rounds]);
-  r1 = _mm_aesenclast_si128 (r1, rk[rounds]);
-  r2 = _mm_aesenclast_si128 (r2, rk[rounds]);
-  r3 = _mm_aesenclast_si128 (r3, rk[rounds]);
-  r4 = _mm_aesenclast_si128 (r4, rk[rounds]);
-  r5 = _mm_aesenclast_si128 (r5, rk[rounds]);
-  r6 = _mm_aesenclast_si128 (r6, rk[rounds]);
-  r7 = _mm_aesenclast_si128 (r7, rk[rounds]);
+  rk = _mm_loadu_si128 ((__m128i*) rk0 + rounds);
+  r0 = _mm_aesenclast_si128 (r0, rk);
+  r1 = _mm_aesenclast_si128 (r1, rk);
+  r2 = _mm_aesenclast_si128 (r2, rk);
+  r3 = _mm_aesenclast_si128 (r3, rk);
+  r4 = _mm_aesenclast_si128 (r4, rk);
+  r5 = _mm_aesenclast_si128 (r5, rk);
+  r6 = _mm_aesenclast_si128 (r6, rk);
+  r7 = _mm_aesenclast_si128 (r7, rk);
 
   _mm_storeu_si128 (out    , r0);
   _mm_storeu_si128 (out + 1, r1);
@@ -257,7 +265,7 @@ static inline void _mc_aesni_dec8 (const uint8_t src[128], uint8_t dst[128], con
 
   __m128i *in  = (__m128i*) src,
           *out = (__m128i*) dst,
-          *rk  = __rk (rk0);
+          rk;
 
   __m128i r0 = _mm_loadu_si128 (in    ),
           r1 = _mm_loadu_si128 (in + 1),
@@ -268,34 +276,37 @@ static inline void _mc_aesni_dec8 (const uint8_t src[128], uint8_t dst[128], con
           r6 = _mm_loadu_si128 (in + 6),
           r7 = _mm_loadu_si128 (in + 7);
 
-  r0 = _mm_xor_si128 (r0, rk[0]);
-  r1 = _mm_xor_si128 (r1, rk[0]);
-  r2 = _mm_xor_si128 (r2, rk[0]);
-  r3 = _mm_xor_si128 (r3, rk[0]);
-  r4 = _mm_xor_si128 (r4, rk[0]);
-  r5 = _mm_xor_si128 (r5, rk[0]);
-  r6 = _mm_xor_si128 (r6, rk[0]);
-  r7 = _mm_xor_si128 (r7, rk[0]);
+  rk = _mm_loadu_si128 ((__m128i*) rk0 + 0);
+  r0 = _mm_xor_si128 (r0, rk);
+  r1 = _mm_xor_si128 (r1, rk);
+  r2 = _mm_xor_si128 (r2, rk);
+  r3 = _mm_xor_si128 (r3, rk);
+  r4 = _mm_xor_si128 (r4, rk);
+  r5 = _mm_xor_si128 (r5, rk);
+  r6 = _mm_xor_si128 (r6, rk);
+  r7 = _mm_xor_si128 (r7, rk);
 
   for (uint8_t i = 1; i < rounds; i++) {
-    r0 = _mm_aesdec_si128 (r0, rk[i]);
-    r1 = _mm_aesdec_si128 (r1, rk[i]);
-    r2 = _mm_aesdec_si128 (r2, rk[i]);
-    r3 = _mm_aesdec_si128 (r3, rk[i]);
-    r4 = _mm_aesdec_si128 (r4, rk[i]);
-    r5 = _mm_aesdec_si128 (r5, rk[i]);
-    r6 = _mm_aesdec_si128 (r6, rk[i]);
-    r7 = _mm_aesdec_si128 (r7, rk[i]);
+    rk = _mm_loadu_si128 ((__m128i*) rk0 + i);
+    r0 = _mm_aesdec_si128 (r0, rk);
+    r1 = _mm_aesdec_si128 (r1, rk);
+    r2 = _mm_aesdec_si128 (r2, rk);
+    r3 = _mm_aesdec_si128 (r3, rk);
+    r4 = _mm_aesdec_si128 (r4, rk);
+    r5 = _mm_aesdec_si128 (r5, rk);
+    r6 = _mm_aesdec_si128 (r6, rk);
+    r7 = _mm_aesdec_si128 (r7, rk);
   }
 
-  r0 = _mm_aesdeclast_si128 (r0, rk[rounds]);
-  r1 = _mm_aesdeclast_si128 (r1, rk[rounds]);
-  r2 = _mm_aesdeclast_si128 (r2, rk[rounds]);
-  r3 = _mm_aesdeclast_si128 (r3, rk[rounds]);
-  r4 = _mm_aesdeclast_si128 (r4, rk[rounds]);
-  r5 = _mm_aesdeclast_si128 (r5, rk[rounds]);
-  r6 = _mm_aesdeclast_si128 (r6, rk[rounds]);
-  r7 = _mm_aesdeclast_si128 (r7, rk[rounds]);
+  rk = _mm_loadu_si128 ((__m128i*) rk0 + rounds);
+  r0 = _mm_aesdeclast_si128 (r0, rk);
+  r1 = _mm_aesdeclast_si128 (r1, rk);
+  r2 = _mm_aesdeclast_si128 (r2, rk);
+  r3 = _mm_aesdeclast_si128 (r3, rk);
+  r4 = _mm_aesdeclast_si128 (r4, rk);
+  r5 = _mm_aesdeclast_si128 (r5, rk);
+  r6 = _mm_aesdeclast_si128 (r6, rk);
+  r7 = _mm_aesdeclast_si128 (r7, rk);
 
   _mm_storeu_si128 (out    , r0);
   _mm_storeu_si128 (out + 1, r1);
@@ -361,23 +372,23 @@ mc_aes_rk_size (value rounds) {
 }
 
 CAMLprim value
-mc_aes_derive_e_key (value key, value off1, value rk, value rounds) {
+mc_aes_derive_e_key (value key, value rk, value rounds) {
   _mc_switch_accel(aesni,
-    mc_aes_derive_e_key_generic(key, off1, rk, rounds),
-    _mc_aesni_derive_e_key (_ba_uint8_off (key, off1),
-                            _ba_uint8 (rk),
+    mc_aes_derive_e_key_generic(key, rk, rounds),
+    _mc_aesni_derive_e_key (_st_uint8 (key),
+                            _bp_uint8 (rk),
                             Int_val (rounds)))
   return Val_unit;
 }
 
 CAMLprim value
-mc_aes_derive_d_key (value key, value off1, value kr, value rounds, value rk) {
+mc_aes_derive_d_key (value key, value kr, value rounds, value rk) {
   _mc_switch_accel(aesni,
-    mc_aes_derive_d_key_generic(key, off1, kr, rounds, rk),
-    _mc_aesni_derive_d_key (_ba_uint8_off (key, off1),
-                            _ba_uint8 (kr),
+    mc_aes_derive_d_key_generic(key, kr, rounds, rk),
+    _mc_aesni_derive_d_key (_st_uint8 (key),
+                            _bp_uint8 (kr),
                             Int_val (rounds),
-                            Is_block(rk) ? _ba_uint8(Field(rk, 0)) : 0))
+                            Is_block(rk) ? _bp_uint8(Field(rk, 0)) : 0))
   return Val_unit;
 }
 
@@ -385,9 +396,9 @@ CAMLprim value
 mc_aes_enc (value src, value off1, value dst, value off2, value rk, value rounds, value blocks) {
   _mc_switch_accel(aesni,
     mc_aes_enc_generic(src, off1, dst, off2, rk, rounds, blocks),
-    _mc_aesni_enc_blocks ( _ba_uint8_off (src, off1),
-                           _ba_uint8_off (dst, off2),
-                           _ba_uint8 (rk),
+    _mc_aesni_enc_blocks ( _st_uint8_off (src, off1),
+                           _bp_uint8_off (dst, off2),
+                           _st_uint8 (rk),
                            Int_val (rounds),
                            Int_val (blocks) ))
   return Val_unit;
@@ -397,9 +408,9 @@ CAMLprim value
 mc_aes_dec (value src, value off1, value dst, value off2, value rk, value rounds, value blocks) {
   _mc_switch_accel(aesni,
     mc_aes_dec_generic(src, off1, dst, off2, rk, rounds, blocks),
-    _mc_aesni_dec_blocks ( _ba_uint8_off (src, off1),
-                           _ba_uint8_off (dst, off2),
-                           _ba_uint8 (rk),
+    _mc_aesni_dec_blocks ( _st_uint8_off (src, off1),
+                           _bp_uint8_off (dst, off2),
+                           _st_uint8 (rk),
                            Int_val (rounds),
                            Int_val (blocks) ))
   return Val_unit;
