@@ -4,6 +4,90 @@ open Mirage_crypto
 
 open Test_common
 
+let des_ecb_cases =
+  let case ~data ~key ~out = vx data, DES.ECB.of_secret (vx key), vx out
+
+  and check (data, key, out) _ =
+    let enc = DES.ECB.encrypt ~key data in
+    let dec = DES.ECB.decrypt ~key enc in
+    assert_oct_equal ~msg:"ciphertext" out enc ;
+    assert_oct_equal ~msg:"plaintext" data dec in
+
+  cases_of check [
+    case
+      ~data:"3f87 9123 0058 8d88  e784 d52a 5d0f 2038
+             f523 6889 bbce ce1f  a7bf 7aa8 6fcc 8245
+             0576 2144 8f11 94d7  07bc 1bba 9b92 5e45
+             3190 c42b 758f 3d91  f68e ebbb ce62 b8e7"
+      ~key: "3f47 f79c c120 7188  4700 217e fd88 bbe4 6f51 27fb 7340 81e5"
+      ~out: "b43b 3ae3 d765 b299  06ea 7c35 ceeb 9e52
+             946c 06e7 0d50 193e  5a22 1ff0 afe9 abe0
+             3b82 ce7d c42a 465d  0f19 45f0 5382 7006
+             b4cd 21f0 5b0f 6843  de2a 67b6 9fb4 6a8f"
+]
+
+let des_cbc_cases =
+  let case ~data ~key ~iv ~out = vx data, DES.CBC.of_secret (vx key), vx iv, vx out
+
+  and check (data, key, iv, out) _ =
+    let enc = DES.CBC.encrypt ~key ~iv data in
+    let dec = DES.CBC.decrypt ~key ~iv enc in
+    assert_oct_equal ~msg:"ciphertext" out enc ;
+    assert_oct_equal ~msg:"plaintext" data dec in
+
+  cases_of check [
+    case
+      ~data:
+"8f8c 1e0a c8fb 1614  3cec ed1c 28ac fd6f
+ae6d 3686 5365 511d  6707 68d9 7928 0479
+cacd 6808 1540 d5fc  2971 2a8a c2b1 17c2
+f0e6 a329 e190 44ff  54e7 5eec 8296 6a58"
+      ~iv:"b219 ef93 4c37 aadf"
+      ~key:"7ecd 2240 a2ac a10a  e713 f467 7ea5 d327  e04c cfe0 5cb4 bb09"
+~out:
+"3110 3904 faa1 4ef4  e404 d3d0 f2ee ae58
+5fe9 e6b7 9552 b4f4  3608 03ca 395a f6e9
+2330 69d6 2c6f a52a  d083 faab 3306 b794
+89f6 6671 e3dd 3368  0b13 f8d9 7136 9674"
+  ]
+
+let des_ctr_cases =
+  let case ~data ~key ~ctr ~out = test_case @@ fun _ ->
+    let open DES.CTR in
+    let key  = vx key |> of_secret
+    and ctr  = vx ctr |> ctr_of_octets
+    and out  = vx out
+    and data = vx data in
+    let enc = encrypt ~key ~ctr data in
+    let dec = decrypt ~key ~ctr enc in
+    assert_oct_equal ~msg:"cipher" out enc;
+    assert_oct_equal ~msg:"plain" data dec
+  in
+  [ case
+      ~data:
+"e9ee ce61 7b75 4c70  79f3 3e5b 036a 7d5b
+4bee f693 0eb3 fa50  9fe3 61d8 713a a487
+a692 21b0 8627 5e6f  d021 4030 7c58 507a
+5fea ca64 d17d a493  7337 8c17 ae05 f3c4
+c6dc 15cc 49c4 3ab0  dab3 9c9b e964 a3c8
+5865 7bb8 6e4d 8507  3866 b805 02c2 4970
+dbbd 3554 20b1 76b2  ee6c 98b3 f7ce 9035
+1e5f 880e"
+~key:"76b9 d4ff d52f 5024  6d24 a3e1 4ebd e605  b82c d81f 0c07 2da1"
+~ctr:"6318 a132 cafd aac0"
+~out:
+"b8d8 aeec d583 009c  f042 ec4d 7ddf c5e5
+386f 89e6 d975 02bc  7583 e113 4899 dabc
+bd93 871b 774b e5ce  4e12 6778 f208 0c53
+52cb a3ac 7567 cdb9  ae81 fc46 25d4 7f9d
+6f3f fbec 4512 8845  3739 1014 2b39 d293
+845a 8505 91a6 f644  5168 bf00 ca4d 4603
+6e5f 418f c43f fabd  272e 1009 c69b 2a6b
+7d2c edb2"
+
+  ]
+
+
 (* NIST SP 800-38A test vectors for block cipher modes of operation *)
 
 let nist_sp_800_38a = vx
@@ -13,15 +97,13 @@ let nist_sp_800_38a = vx
    f6 9f 24 45 df 4f 9b 17 ad 2b 41 7b e6 6c 37 10"
 
 let aes_ecb_cases =
-  let open Cipher_block in
-
   let case ~key ~out = (AES.ECB.of_secret (vx key), vx out)
 
   and check (key, out) _ =
     let enc = AES.ECB.encrypt ~key nist_sp_800_38a in
     let dec = AES.ECB.decrypt ~key enc in
-    assert_cs_equal ~msg:"ciphertext" out enc ;
-    assert_cs_equal ~msg:"plaintext" nist_sp_800_38a dec in
+    assert_oct_equal ~msg:"ciphertext" out enc ;
+    assert_oct_equal ~msg:"plaintext" nist_sp_800_38a dec in
 
   cases_of check [
     case ~key: "2b 7e 15 16 28 ae d2 a6 ab f7 15 88 09 cf 4f 3c"
@@ -46,15 +128,13 @@ let aes_ecb_cases =
   ]
 
 let aes_cbc_cases =
-  let open Cipher_block in
-
   let case ~key ~iv ~out = (AES.CBC.of_secret (vx key), vx iv, vx out)
 
   and check (key, iv, out) _ =
     let enc = AES.CBC.encrypt ~key ~iv nist_sp_800_38a in
     let dec = AES.CBC.decrypt ~key ~iv enc in
-    assert_cs_equal ~msg:"ciphertext" out enc ;
-    assert_cs_equal ~msg:"plaintext" nist_sp_800_38a dec in
+    assert_oct_equal ~msg:"ciphertext" out enc ;
+    assert_oct_equal ~msg:"plaintext" nist_sp_800_38a dec in
 
   cases_of check [
     case ~key: "2b 7e 15 16 28 ae d2 a6 ab f7 15 88 09 cf 4f 3c"
@@ -83,16 +163,16 @@ let aes_cbc_cases =
 
 let aes_ctr_cases =
   let case ~key ~ctr ~out ~ctr1 = test_case @@ fun _ ->
-    let open Cipher_block.AES.CTR in
+    let open AES.CTR in
     let key  = vx key |> of_secret
-    and ctr  = vx ctr |> ctr_of_cstruct
-    and ctr1 = vx ctr1 |> ctr_of_cstruct
+    and ctr  = vx ctr |> ctr_of_octets
+    and ctr1 = vx ctr1 |> ctr_of_octets
     and out  = vx out in
     let enc = encrypt ~key ~ctr nist_sp_800_38a in
     let dec = decrypt ~key ~ctr enc in
-    assert_cs_equal ~msg:"cipher" out enc;
-    assert_cs_equal ~msg:"plain" nist_sp_800_38a dec;
-    let blocks = Cstruct.length nist_sp_800_38a / block_size in
+    assert_oct_equal ~msg:"cipher" out enc;
+    assert_oct_equal ~msg:"plain" nist_sp_800_38a dec;
+    let blocks = String.length nist_sp_800_38a / block_size in
     assert_equal ~msg:"counters" ctr1 (add_ctr ctr (Int64.of_int blocks))
   in
   [ case ~key:  "2b7e1516 28aed2a6 abf71588 09cf4f3c"
@@ -133,8 +213,6 @@ let aes_ctr_cases =
 (* aes gcm *)
 
 let gcm_cases =
-  let open Cipher_block in
-
   let case ~key ~p ~a ~nonce ~c ~t =
     (AES.GCM.of_secret (vx key), vx p, vx a, vx nonce, vx c, vx t) in
 
@@ -145,8 +223,8 @@ let gcm_cases =
       | None -> assert_failure "GCM decryption broken"
       | Some data -> data
     in
-    assert_cs_equal ~msg:"ciphertext" (Cstruct.append c t) cipher ;
-    assert_cs_equal ~msg:"decrypted plaintext" p pdata
+    assert_oct_equal ~msg:"ciphertext" (c ^ t) cipher ;
+    assert_oct_equal ~msg:"decrypted plaintext" p pdata
   in
 
   cases_of check [
@@ -292,9 +370,9 @@ let ccm_cases =
 
   let check (key, p, adata, nonce, c) _ =
     let cip = authenticate_encrypt ~key ~nonce ~adata p in
-    assert_cs_equal ~msg:"encrypt" c cip ;
+    assert_oct_equal ~msg:"encrypt" c cip ;
     match authenticate_decrypt ~key ~nonce ~adata c with
-      | Some x -> assert_cs_equal ~msg:"decrypt" p x
+      | Some x -> assert_oct_equal ~msg:"decrypt" p x
       | None -> assert_failure "CCM decryption broken"
   in
 
@@ -324,24 +402,24 @@ let ccm_cases =
 *)
 
 let ccm_regressions =
-  let open Cipher_block.AES.CCM16 in
+  let open AES.CCM16 in
   let no_vs_empty_ad _ =
     (* as reported in https://github.com/mirleft/ocaml-nocrypto/issues/166 *)
     (* see RFC 3610 Section 2.1, AD of length 0 should be same as no AD *)
     let key = of_secret (vx "000102030405060708090a0b0c0d0e0f")
     and nonce = vx "0001020304050607"
-    and plaintext = Cstruct.of_string "hello"
+    and plaintext = "hello"
     in
-    assert_cs_equal ~msg:"CCM no vs empty ad"
+    assert_oct_equal ~msg:"CCM no vs empty ad"
       (authenticate_encrypt ~key ~nonce plaintext)
-      (authenticate_encrypt ~adata:Cstruct.empty ~key ~nonce plaintext)
+      (authenticate_encrypt ~adata:"" ~key ~nonce plaintext)
   and short_nonce_enc _ =
     (* as reported in https://github.com/mirleft/ocaml-nocrypto/issues/167 *)
     (* valid nonce sizes for CCM are 7..13 (L can be 2..8, nonce is 15 - L)*)
     (* see RFC3610 Section 2.1 *)
     let key = of_secret (vx "000102030405060708090a0b0c0d0e0f")
-    and nonce = Cstruct.empty
-    and plaintext = Cstruct.of_string "hello"
+    and nonce = ""
+    and plaintext = "hello"
     in
     assert_raises ~msg:"CCM with short nonce raises"
       (Invalid_argument "Mirage_crypto: CCM: nonce length not between 7 and 13: 0")
@@ -349,7 +427,7 @@ let ccm_regressions =
   and short_nonce_enc2 _ =
     let key = of_secret (vx "000102030405060708090a0b0c0d0e0f")
     and nonce = vx "00"
-    and plaintext = Cstruct.of_string "hello"
+    and plaintext = "hello"
     in
     assert_raises ~msg:"CCM with short nonce raises"
       (Invalid_argument "Mirage_crypto: CCM: nonce length not between 7 and 13: 1")
@@ -357,7 +435,7 @@ let ccm_regressions =
   and short_nonce_enc3 _ =
     let key = of_secret (vx "000102030405060708090a0b0c0d0e0f")
     and nonce = vx "000102030405"
-    and plaintext = Cstruct.of_string "hello"
+    and plaintext = "hello"
     in
     assert_raises ~msg:"CCM with short nonce raises"
       (Invalid_argument "Mirage_crypto: CCM: nonce length not between 7 and 13: 6")
@@ -365,7 +443,7 @@ let ccm_regressions =
   and long_nonce_enc _ =
     let key = of_secret (vx "000102030405060708090a0b0c0d0e0f")
     and nonce = vx "000102030405060708090a0b0c0d"
-    and plaintext = Cstruct.of_string "hello"
+    and plaintext = "hello"
     in
     assert_raises ~msg:"CCM with short nonce raises"
       (Invalid_argument "Mirage_crypto: CCM: nonce length not between 7 and 13: 14")
@@ -374,13 +452,175 @@ let ccm_regressions =
     (* as reported in https://github.com/mirleft/ocaml-nocrypto/issues/168 *)
     let key = of_secret (vx "000102030405060708090a0b0c0d0e0f")
     and nonce = vx "0001020304050607"
-    and adata = Cstruct.of_string "hello"
-    and p = Cstruct.empty
+    and adata = "hello"
+    and p = ""
     in
     let cipher = authenticate_encrypt ~adata ~key ~nonce p in
     match authenticate_decrypt ~key ~nonce ~adata cipher with
-    | Some x -> assert_cs_equal ~msg:"CCM decrypt of empty message" p x
+    | Some x -> assert_oct_equal ~msg:"CCM decrypt of empty message" p x
     | None -> assert_failure "decryption broken"
+  and long_adata _ =
+    let key = of_secret (vx "000102030405060708090a0b0c0d0e0f")
+    and nonce = vx "0001020304050607"
+    and plaintext = "hello"
+    (* [adata] is greater than [1 lsl 16 - 1 lsl 8] *)
+    and adata = String.make 65280 '\x00'
+    and expected = vx "6592169e946f98973bc06d080f7c9dbb493a536f8a"
+    in
+    let cipher = authenticate_encrypt ~adata ~key ~nonce plaintext in
+    assert_oct_equal ~msg:"CCM encrypt of >=65280 adata" expected cipher
+  in
+  let regr_tls =
+    let key = of_secret (vx "063a 96fd 15f9 82d5  5aad 5bf9 d098 7546") in
+    (* discovered while moving ocaml-tls to string *)
+    let nonce = vx "81cd 4758 1880 9de0  c655 7c31"
+    and adata = vx "1703 0300 17"
+    and data = vx "0800 0002 0000 16"
+    and expected = vx "94ca 065a c948 c5d6  92fd 5fab c850 0611  a07c 4f6e 0710 90"
+    in
+    let a _ =
+      let cipher = authenticate_encrypt ~adata ~key ~nonce data in
+      assert_oct_equal ~msg:"TLS regression 0" expected cipher
+    and b _ =
+      match authenticate_decrypt ~key ~nonce ~adata expected with
+      | None -> assert_failure "TLS regression 0, decrypt broken"
+      | Some x -> assert_oct_equal ~msg:"TLS regression 0 decrypt" x data
+    in
+    let nonce = vx "81cd 4758 1880 9de0  c655 7c30"
+    and adata = vx "1703 0302 85"
+    and data = vx {|
+0b00 0270 0000 026c  0002 6730 8202 6330
+8201 cc02 0900 cb6c  4e84 4b58 a1d4 300d
+|}
+    and expected = vx {|
+1e59 904e e6d5 c2ac  e538 78d7 e24f 6e46
+6169 f8e2 d3dd 8b5d  788c 78ff ea9f e1d0
+9885 ac1a c6d9 fb88  b66a 3a11 5ba5 6e7c
+        |}
+    in
+    let c _ =
+      let cipher = authenticate_encrypt ~adata ~key ~nonce data in
+      assert_oct_equal ~msg:"TLS regression 1" expected cipher
+    and d _ =
+      match authenticate_decrypt ~key ~nonce ~adata expected with
+      | None -> assert_failure "TLS regression 1, decrypt broken"
+      | Some x -> assert_oct_equal ~msg:"TLS regression 1 decrypt" x data
+    in
+    let data = vx {|
+0b00 0270 0000 026c  0002 6730 8202 6330
+8201 cc02 0900 cb6c  4e84 4b58 a1d4 300d
+8201 cc02 0900
+|}
+    and expected = vx {|
+1e59 904e e6d5 c2ac  e538 78d7 e24f 6e46
+6169 f8e2 d3dd 8b5d  788c 78ff ea9f e1d0
+7c8d 9993 6bfd cf76  9799 473b 58f4 ed69
+d7a4 df7a 2d6b
+      |}
+    in
+    let e _ =
+      let cipher = authenticate_encrypt ~adata ~key ~nonce data in
+      assert_oct_equal ~msg:"TLS regression 2" expected cipher
+    and f _ =
+      match authenticate_decrypt ~key ~nonce ~adata expected with
+      | None -> assert_failure "TLS regression 2, decrypt broken"
+      | Some x -> assert_oct_equal ~msg:"TLS regression 2 decrypt" x data
+    in
+    let data = vx {|
+0b00 0270 0000 026c  0002 6730 8202 6330
+8201 cc02 0900 cb6c  4e84 4b58 a1d4 300d
+0609 2a86 4886 f70d  0101 0505 0030 7631
+0b30 0906 0355 0406  1302 4155 3113 3011
+0603 5504 080c 0a53  6f6d 652d 5374 6174
+6531 2130 1f06 0355  040a 0c18 496e 7465
+726e 6574 2057 6964  6769 7473 2050 7479
+204c 7464 3115 3013  0603 5504 030c 0c59
+4f55 5220 4e41 4d45  2121 2131 1830 1606
+092a 8648 86f7 0d01  0901 1609 6d65 4062
+6172 2e64 6530 1e17  0d31 3430 3231 3732
+3230 3834 355a 170d  3135 3032 3137 3232
+3038 3435 5a30 7631  0b30 0906 0355 0406
+1302 4155 3113 3011  0603 5504 080c 0a53
+6f6d 652d 5374 6174  6531 2130 1f06 0355
+040a 0c18 496e 7465  726e 6574 2057 6964
+6769 7473 2050 7479  204c 7464 3115 3013
+0603 5504 030c 0c59  4f55 5220 4e41 4d45
+2121 2131 1830 1606  092a 8648 86f7 0d01
+0901 1609 6d65 4062  6172 2e64 6530 819f
+300d 0609 2a86 4886  f70d 0101 0105 0003
+818d 0030 8189 0281  8100 b640 48de e6bc
+2194 3da2 ab5e b6f8  d837 007f 417c 0fe3
+3492 c3aa 2f55 3e4d  5e31 4346 89c2 6f2b
+e68e 00d2 88b0 e3ab  f6fe 1188 45d9 4989
+8512 f192 cbe4 9fd5  b083 1f01 cb2d 274d
+b3a6 38f5 befb 3ce8  1ab6 b559 3934 4404
+4fed d6ca 154f 76bf  bd52 5608 bb55 0a39
+bbd2 ed12 e6d7 1f9f  84ba 21aa 5e21 8015
+0267 1aab 049a f864  0da1 0203 0100 0130
+0d06 092a 8648 86f7  0d01 0105 0500 0381
+8100 8a38 669a 4896  9dc9 4729 6d44 2d7f
+0320 82d2 db21 e537  4cdd 6ef6 e7cc 1da0
+fde5 11ed 3c52 52f0  a673 dc68 9fdc 5fca
+cc1b 85df e22b 7bef  2adb 56b5 3732 e981
+1063 794d 6e23 9f8f  a267 215b a7a4 d3dc
+e505 e799 ec5c 38cd  1c16 ee75 e0d5 a46b
+8f4c 8e82 6505 6153  9a84 305d f19a 5a24
+1be5 55f8 7083 4e09  4d41 cf9f 74b3 342e
+8345 0000 16
+        |}
+    and expected = vx {|
+1e59 904e e6d5 c2ac  e538 78d7 e24f 6e46
+6169 f8e2 d3dd 8b5d  788c 78ff ea9f e1d0
+f885 7f17 2a7b f163  d29e 0a8e 8636 418f
+a9da 651b f2ba 36aa  a1a4 14d0 6a9a f991
+0836 eb93 80b9 bbe2  1f20 98d9 be0b c16f
+d58c c98d 4082 dadd  f575 57a4 43f7 af31
+c1b7 1eeb 2590 a887  e31c 590a 7e56 798c
+69aa 4576 fde6 63d2  1b62 d00d 98f6 4015
+dae7 8454 b96a f7f9  774f f539 24bf efe6
+4629 ee35 4c81 32d4  43df ffa9 17a2 6306
+fd07 f9ab b462 2bcd  bb0a 3750 af1a 3525
+66ad 6c67 b647 2ca7  d6b5 b13e ea34 d90d
+5731 a599 e608 d037  bc77 40aa b305 84ad
+8d78 43fc 7f55 70a2  fbbb 1b30 a14a 2f5f
+b3c3 2584 1f9e 7f3f  3dfa 19e2 9539 a1be
+ead8 e051 d847 915b  ed23 87ab 7082 7df4
+71a0 e0a6 46db a780  1e7b fb98 dac4 0af1
+c3eb 42d4 3a6c 3c71  f55a b377 e4de ff20
+14d7 b47c 8743 f291  56f3 6d8c 45d1 7cb3
+0321 e2cf 8ffd babf  a129 ea0d cc1b 7a0d
+b1ec 448d 0e3b 4386  9cc2 2b5a 5569 2930
+ea33 080e 9168 3696  b224 6238 34fc 3e25
+7895 6af3 cd60 f3c8  6643 3d6f 5736 4e78
+6aca 8b2d 1575 2d34  4533 79bd e27e 9c46
+f9f4 be4a 2fe3 f377  3acf 7b6e e4f0 3eb0
+ec85 95a6 ed04 2316  fe4e 2a54 25aa c40a
+c464 4128 0e35 1003  9f5d abfa e8e9 dc73
+f709 f29b f930 0bdc  d941 981b c5b3 8295
+97a5 c7e9 481d ce99  c6b6 5dfb 672d 3fdb
+38bb a6be d7f8 9863  345d c3a8 77f3 6b77
+f309 5c3b b9df fa40  8d42 ff79 6724 23da
+8f24 c9b0 e02d 4794  581f e185 32e6 94bb
+5b6a 6d5c 3b80 4c83  a0d8 0b42 d575 4fc3
+4353 a78d fdb5 003c  4f0b 437d 75fb 5886
+a76a 35f5 892d a10b  ce33 3ce6 ffd9 f09c
+7264 5b09 c50a 7013  344c 11a1 ab92 5728
+43e1 bc8c 8c1b 3fad  4a02 25a9 cb96 5fd2
+1962 4b0c b46b 9f8f  1225 b18c 2572 6297
+c890 238f 22d6 2bb0  7678 568a 3c9b 75e5
+b8fc 10f3 13c7 aa16  8165 a29c 67f1 46f4
+6e44 8e84 f5
+|}
+    in
+    let g _ =
+      let cipher = authenticate_encrypt ~adata ~key ~nonce data in
+      assert_oct_equal ~msg:"TLS regression 3" expected cipher
+    and h _ =
+      match authenticate_decrypt ~key ~nonce ~adata expected with
+      | None -> assert_failure "TLS regression 3, decrypt broken"
+      | Some x -> assert_oct_equal ~msg:"TLS regression 3 decrypt" x data
+    in
+    [ a ; b ; c ; d ; e ; f ; g ; h ]
   in
   [
     test_case no_vs_empty_ad ;
@@ -389,13 +629,14 @@ let ccm_regressions =
     test_case short_nonce_enc3 ;
     test_case long_nonce_enc ;
     test_case enc_dec_empty_message ;
-  ]
+    test_case long_adata ;
+  ] @ List.map test_case regr_tls
 
 let gcm_regressions =
-  let open Cipher_block.AES.GCM in
+  let open AES.GCM in
   let msg = vx "000102030405060708090a0b0c0d0e0f" in
   let key = of_secret msg
-  and nonce = Cstruct.empty
+  and nonce = ""
   in
   let nonce_zero_length_enc _ =
     (* reported in https://github.com/mirleft/ocaml-nocrypto/issues/169 *)
@@ -406,37 +647,22 @@ let gcm_regressions =
     assert_raises ~msg:"GCM with nonce of 0"
       (Invalid_argument "Mirage_crypto: GCM: invalid nonce of length 0")
       (fun () -> authenticate_decrypt ~key ~nonce msg)
-  and unaligned _ =
-    let key = of_secret (vx "00000000000000000000000000000000")
-    and c = vx "0388dace60b6a392f328c2b971b2fe78"
-    and p = vx "00000000000000000000000000000000"
-    and nonce = vx "000000000000000000000000"
-    and t = vx "ab6e47d42cec13bdf53a67b21257bddf"
-    in
-    let cipher = Cstruct.shift (Cstruct.concat [ Cstruct.create 1 ; c ; t ]) 1 in
-    let auth_dec ~key ~nonce cipher = match authenticate_decrypt ~key ~nonce cipher with
-      | None -> assert_failure "GCM decryption failure"
-      | Some x -> x
-    in
-    assert_cs_equal ~msg:"GCM with unaligned msg"
-      (auth_dec ~key ~nonce cipher) p
   in
   [
     test_case nonce_zero_length_enc ;
     test_case nonce_zero_length_dec ;
-    test_case unaligned ;
   ]
 
 
 let chacha20_cases =
-  let case msg ?ctr ~key ~nonce ?(input = Cstruct.create 128) output =
+  let case msg ?ctr ~key ~nonce ?(input = String.make 128 '\000') output =
     let key = Chacha20.of_secret (vx key)
     and nonce = vx nonce
     and output = vx output
     in
-    assert_cs_equal ~msg (Chacha20.crypt ~key ~nonce ?ctr input) output
+    assert_oct_equal ~msg (Chacha20.crypt ~key ~nonce ?ctr input) output
   in
-  let rfc8439_input = Cstruct.of_string "Ladies and Gentlemen of the class of '99: If I could offer you only one tip for the future, sunscreen would be it." in
+  let rfc8439_input = "Ladies and Gentlemen of the class of '99: If I could offer you only one tip for the future, sunscreen would be it." in
   let rfc8439_test_2_4_2 _ =
     let key = "000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f"
     and nonce = "000000000000004a00000000"
@@ -466,17 +692,13 @@ let chacha20_cases =
   61 16
   1a e1 0b 59 4f 09 e2 6a 7e 90 2e cb d0 60 06 91|}
     in
-    assert_cs_equal ~msg:"Chacha20/Poly1305 RFC 8439 2.8.2 encrypt"
+    assert_oct_equal ~msg:"Chacha20/Poly1305 RFC 8439 2.8.2 encrypt"
       (Chacha20.authenticate_encrypt ~key ~nonce ~adata rfc8439_input)
       output;
-    assert_cs_equal ~msg:"Chacha20/Poly1305 RFC 8439 2.8.2 decrypt"
+    assert_oct_equal ~msg:"Chacha20/Poly1305 RFC 8439 2.8.2 decrypt"
       (match Chacha20.authenticate_decrypt ~key ~nonce ~adata output with
        | Some cs -> cs | None -> assert_failure "Chacha20/poly1305 decryption broken")
       rfc8439_input;
-    let input = Cstruct.(shift (append (create 16) rfc8439_input) 16) in
-    assert_cs_equal ~msg:"Chacha20/Poly1305 RFC 8439 2.8.2 encrypt 2"
-      (Chacha20.authenticate_encrypt ~key ~nonce ~adata input)
-      output;
   in
   (* from https://tools.ietf.org/html/draft-strombergson-chacha-test-vectors-01 *)
   let case ~key ~nonce ~output0 ~output1 _ =
@@ -683,93 +905,95 @@ let chacha20_cases =
 
 let poly1305_rfc8439_2_5_2 _ =
   let key = vx "85d6be7857556d337f4452fe42d506a80103808afb0db2fd4abff6af4149f51b"
-  and data = Cstruct.of_string "Cryptographic Forum Research Group"
+  and data = "Cryptographic Forum Research Group"
   and output = vx "a8061dc1305136c6c22b8baf0c0127a9"
   in
-  assert_cs_equal ~msg:"poly 1305 RFC8439 Section 2.5.2"
+  assert_oct_equal ~msg:"poly 1305 RFC8439 Section 2.5.2"
     (Poly1305.mac ~key data) output
 
 let empty_cases _ =
-  let open Cipher_block in
-  let plain = Cstruct.empty
-  and cipher = Cstruct.empty
+  let plain = ""
+  and cipher = ""
   in
   (* 3DES ECB CBC CTR *)
   Array.iter (fun key_size ->
-      let key = DES.ECB.of_secret (Cstruct.create key_size) in
-      assert_cs_equal ~msg:"DES ECB encrypt" cipher (DES.ECB.encrypt ~key plain) ;
-      assert_cs_equal ~msg:"DES ECB decrypt" plain (DES.ECB.decrypt ~key cipher))
+      let key = DES.ECB.of_secret (String.make key_size '\x00') in
+      assert_oct_equal ~msg:"DES ECB encrypt" cipher (DES.ECB.encrypt ~key plain) ;
+      assert_oct_equal ~msg:"DES ECB decrypt" plain (DES.ECB.decrypt ~key cipher))
     DES.ECB.key_sizes ;
   Array.iter (fun key_size ->
-      let key = DES.CBC.of_secret (Cstruct.create key_size)
-      and iv = Cstruct.create DES.CBC.block_size
+      let key = DES.CBC.of_secret (String.make key_size '\x00')
+      and iv = String.make DES.CBC.block_size '\x00'
       in
-      assert_cs_equal ~msg:"DES CBC encrypt" cipher (DES.CBC.encrypt ~key ~iv plain) ;
-      assert_cs_equal ~msg:"DES CBC decrypt" plain (DES.CBC.decrypt ~key ~iv cipher))
+      assert_oct_equal ~msg:"DES CBC encrypt" cipher (DES.CBC.encrypt ~key ~iv plain) ;
+      assert_oct_equal ~msg:"DES CBC decrypt" plain (DES.CBC.decrypt ~key ~iv cipher))
     DES.CBC.key_sizes ;
   Array.iter (fun key_size ->
-      let key = DES.CTR.of_secret (Cstruct.create key_size)
-      and ctr = DES.CTR.ctr_of_cstruct (Cstruct.create DES.CTR.block_size)
+      let key = DES.CTR.of_secret (String.make key_size '\x00')
+      and ctr = DES.CTR.ctr_of_octets (String.make DES.CTR.block_size '\x00')
       in
-      assert_cs_equal ~msg:"DES CTR encrypt" cipher (DES.CTR.encrypt ~key ~ctr plain) ;
-      assert_cs_equal ~msg:"DES CTR decrypt" plain (DES.CTR.decrypt ~key ~ctr cipher))
+      assert_oct_equal ~msg:"DES CTR encrypt" cipher (DES.CTR.encrypt ~key ~ctr plain) ;
+      assert_oct_equal ~msg:"DES CTR decrypt" plain (DES.CTR.decrypt ~key ~ctr cipher))
     DES.CTR.key_sizes ;
 
   (* AES ECB CBC CTR GCM CCM16 *)
   Array.iter (fun key_size ->
-      let key = AES.ECB.of_secret (Cstruct.create key_size) in
-      assert_cs_equal ~msg:"AES ECB encrypt" cipher (AES.ECB.encrypt ~key plain) ;
-      assert_cs_equal ~msg:"AES ECB decrypt" plain (AES.ECB.decrypt ~key cipher))
+      let key = AES.ECB.of_secret (String.make key_size '\x00') in
+      assert_oct_equal ~msg:"AES ECB encrypt" cipher (AES.ECB.encrypt ~key plain) ;
+      assert_oct_equal ~msg:"AES ECB decrypt" plain (AES.ECB.decrypt ~key cipher))
     AES.ECB.key_sizes ;
   Array.iter (fun key_size ->
-      let key = AES.CBC.of_secret (Cstruct.create key_size)
-      and iv = Cstruct.create AES.CBC.block_size
+      let key = AES.CBC.of_secret (String.make key_size '\x00')
+      and iv = String.make AES.CBC.block_size '\x00'
       in
-      assert_cs_equal ~msg:"AES CBC encrypt" cipher (AES.CBC.encrypt ~key ~iv plain) ;
-      assert_cs_equal ~msg:"AES CBC decrypt" plain (AES.CBC.decrypt ~key ~iv cipher))
+      assert_oct_equal ~msg:"AES CBC encrypt" cipher (AES.CBC.encrypt ~key ~iv plain) ;
+      assert_oct_equal ~msg:"AES CBC decrypt" plain (AES.CBC.decrypt ~key ~iv cipher))
     AES.CBC.key_sizes ;
   Array.iter (fun key_size ->
-      let key = AES.CTR.of_secret (Cstruct.create key_size)
-      and ctr = AES.CTR.ctr_of_cstruct (Cstruct.create AES.CTR.block_size)
+      let key = AES.CTR.of_secret (String.make key_size '\x00')
+      and ctr = AES.CTR.ctr_of_octets (String.make AES.CTR.block_size '\x00')
       in
-      assert_cs_equal ~msg:"AES CTR encrypt" cipher (AES.CTR.encrypt ~key ~ctr plain) ;
-      assert_cs_equal ~msg:"AES CTR decrypt" plain (AES.CTR.decrypt ~key ~ctr cipher))
+      assert_oct_equal ~msg:"AES CTR encrypt" cipher (AES.CTR.encrypt ~key ~ctr plain) ;
+      assert_oct_equal ~msg:"AES CTR decrypt" plain (AES.CTR.decrypt ~key ~ctr cipher))
     AES.CTR.key_sizes ;
   Array.iter (fun key_size ->
-      let key = AES.CCM16.of_secret (Cstruct.create key_size) in
+      let key = AES.CCM16.of_secret (String.make key_size '\x00') in
       let test_one nonce =
         let c, tag = AES.CCM16.authenticate_encrypt_tag ~key ~nonce plain in
-        assert_cs_equal ~msg:"AES CCM16 encrypt" cipher c ;
+        assert_oct_equal ~msg:"AES CCM16 encrypt" cipher c ;
         match AES.CCM16.authenticate_decrypt_tag ~key ~nonce ~tag cipher with
         | None -> assert false
-        | Some p -> assert_cs_equal ~msg:"AES CCM16 decrypt" plain p
+        | Some p -> assert_oct_equal ~msg:"AES CCM16 decrypt" plain p
       in
-      test_one (Cstruct.create 7);
-      test_one (Cstruct.create 8);
-      test_one (Cstruct.create 13))
+      test_one (String.make 7 '\x00');
+      test_one (String.make 8 '\x00');
+      test_one (String.make 13 '\x00'))
     AES.CCM16.key_sizes ;
 
   (* ChaCha20 *)
   Array.iter (fun key_size ->
-      let key = Chacha20.of_secret (Cstruct.create key_size) in
+      let key = Chacha20.of_secret (String.make key_size '\x00') in
       let test_one nonce =
         let c, tag = Chacha20.authenticate_encrypt_tag ~key ~nonce plain in
-        assert_cs_equal ~msg:"Chacha20 encrypt" cipher c ;
+        assert_oct_equal ~msg:"Chacha20 encrypt" cipher c ;
         match Chacha20.authenticate_decrypt_tag ~key ~nonce ~tag cipher with
         | None -> assert false
-        | Some p -> assert_cs_equal ~msg:"Chacha20 decrypt" plain p
+        | Some p -> assert_oct_equal ~msg:"Chacha20 decrypt" plain p
       in
-      test_one (Cstruct.create 8);
+      test_one (String.make 8 '\x00');
       if key_size = 32 then
-        test_one (Cstruct.create 12))
+        test_one (String.make 12 '\x00'))
     [| 16 ; 32 |] ;
 
   (* ARC4 *)
-  let key = Cipher_stream.ARC4.of_secret (Cstruct.create 16) in
-  assert_cs_equal ~msg:"ARC4 encrypt" cipher (Cipher_stream.ARC4.(encrypt ~key plain).message) ;
-  assert_cs_equal ~msg:"ARC4 decrypt" plain (Cipher_stream.ARC4.(decrypt ~key cipher).message)
+  let key = ARC4.of_secret (String.make 16 '\x00') in
+  assert_oct_equal ~msg:"ARC4 encrypt" cipher (ARC4.(encrypt ~key plain).message) ;
+  assert_oct_equal ~msg:"ARC4 decrypt" plain (ARC4.(decrypt ~key cipher).message)
 
 let suite = [
+  "3DES-ECB" >::: des_ecb_cases ;
+  "3DES-CBC" >::: des_cbc_cases ;
+  "3DES-CTR" >::: des_ctr_cases ;
   "AES-ECB" >::: [ "SP 300-38A" >::: aes_ecb_cases ] ;
   "AES-CBC" >::: [ "SP 300-38A" >::: aes_cbc_cases ] ;
   "AES-CTR" >::: [ "SP 300-38A" >::: aes_ctr_cases; ] ;
