@@ -1,5 +1,5 @@
 
-let data = ref Cstruct.empty
+let data = ref ""
 
 let cpu_bootstrap_check () =
   match Mirage_crypto_rng.Entropy.cpu_rng_bootstrap with
@@ -12,8 +12,8 @@ let cpu_bootstrap_check () =
       for i = 0 to 10 do
         try
           let data' = cpu_rng_bootstrap 1 in
-          if Cstruct.equal !data data' then begin
-            Cstruct.hexdump data';
+          if String.equal !data data' then begin
+            Ohex.pp Format.std_formatter data';
             failwith ("same data from CPU bootstrap at " ^ string_of_int i);
           end;
           data := data'
@@ -23,8 +23,8 @@ let cpu_bootstrap_check () =
 let whirlwind_bootstrap_check () =
   for i = 0 to 10 do
     let data' = Mirage_crypto_rng.Entropy.whirlwind_bootstrap 1 in
-    if Cstruct.equal !data data' then begin
-      Cstruct.hexdump data';
+    if String.equal !data data' then begin
+      Ohex.pp Format.std_formatter data';
       failwith ("same data from whirlwind bootstrap at " ^ string_of_int i);
     end;
     data := data'
@@ -32,9 +32,9 @@ let whirlwind_bootstrap_check () =
 
 let timer_check () =
   for i = 0 to 10 do
-    let data' = Mirage_crypto_rng.Entropy.interrupt_hook () () in
-    if Cstruct.equal !data data' then begin
-      Cstruct.hexdump data';
+    let data' = Mirage_crypto_rng.Entropy.interrupt_hook () in
+    if String.equal !data data' then begin
+      Ohex.pp Format.std_formatter data';
       failwith ("same data from timer at " ^ string_of_int i);
     end;
     data := data'
